Software for automated data analysis of Wilson_tm fermions with GEVP techniques.

The GEVP is solved as outlined in 1010.0202, i.e., given a correlator f(t) the following matrix is constructed:

```math
C(t) = \bigg[\begin{matrix} f(t) & f(t+\tau) \\ f(t+\tau) & f(t+2\tau)  \end{matrix} \bigg]
```

The code automatically builds these matrices for PP and PA1 correlators  at each time slice and for each \mu sector outlined by the user in the sector dictionary (ll-hl-sh etc).
After this procedure, the generalised eigenvalues are extracted by solving the equation 
```math
C(t,\tau)v(t,t_0,\tau)  = \lambda(t,t_0, \tau) C(t_0, \tau)v(t,t_0,\tau) 
```
The two time parameters \tau and t_0 can be choosen by the user before compiling the code in the setting options.
The  effective energy spectrum is determined from:
```math
E_n^{eff}(t, t_0, \tau) = \log( \frac{\lambda_n(t, t_0, \tau)}{\lambda_n(t+a, t_0, \tau)}) = E_n + O(e^{-(E_m-e_n)t})  
```
Finally the energy levels are determined by fitting the effective energy with the functional form 
```math
E_n^{eff}(t, t_0, \tau) = E_n + p_1*e^{-\Delta E * t}  
```
The fit is performed in the time extent 

[range_t_fit[1]:range_t_fit[2], end-5]

where range_t_fit is an array of integers set by default to [2,5], but this can be changed by the user.
Final results for the energy spectrum is given by the weighthed average of the fit parameters determined by the above outlined procedure.
N.B. at the moment the code does not extract matrix element, hence decay constants, from the gevp.

To run the code in the juliaREPL type:
julia> include("/path_to/gevp.jl")
Go to SET UP VARIABLES section in the gevp.jl file to properly configure before running.
You have to up set the following variables:

- path_data where data are stored
- path_plat where a plat.txt file is stored with plateaus for all ensembles
- path_result where you want to save results.txt and plots
- ensembles to analyse
- sector you are interested in: light-light, light-heavy, heavy-heavy...
- tau value to use for matrix construction 
- _t0 value to use for solving the GEVP
- range_t_fit array of integers with the fit range value for the extractions of the energies 
- rwf set to true if you want to include reweighting factors
- compute_t0 set to true if you want to extract t0 from ms.dat file. Otherwise, t0 is taken from 1608.08900 in const.jl
- mass_shift set to true if you want to include the mass shift
go to line 484 to modify continuum limit + chiral extrapolatio fit model

Data in path_data need to be stored as follow:

../data_folder:

../data_folder/ens_id_1_folder:
- mesons.dat
- ms.dat
- ms1.dat
- pbp.dat


../data_folder/ens_id_2_folder:

- r0_mesons.dat
- r1_mesons.dat
- r0_ms.dat
- r1_ms.dat
- r0_ms1.dat
- r1_ms1.dat
- r0_pbp.dat
- r1_pbp.dat


and so on