
func_map = [Bool.([i,j,k,m,n]) for i=0:1 for j=0:1 for k=0:1 for m=0:1 for n=0:1]

basemodel(x,p) = p[1] .+ p[2] .* x[:,2] .+ p[3] .* x[:,3]  .+ p[4] .* x[:,1]
a2phi2_2(x) = x[:,1] .* x[:,2].^2  #a^2/8t0*phi2^2 
a2phi2(x) = x[:,1] .* x[:,2]      # phi2*a^2/8t0
a2phih2(x) = x[:,1] .* x[:,3].^2  # phih^2*a^2/8t0          
a3phih3(x) = x[:,1].^(4) .* x[:,3].^4  #phih^3*(a^2/8t0)^3/2          
a3cutoff(x) = x[:,1].^(4)   #(a^2/8t0)^4

func_list = [a2phi2_2, a2phi2, a2phih2, a3phih3, a3cutoff]
npar = 5 # number of extra parameters 
n_lin_param = Vector(4:npar+4) #param for each functions
f_lin = Vector{Vector{Function}}(undef, npar+1) #f[i][j]-> i: number of extra parameters, j: combinations
f_lin[1] = Vector{Function}(undef, 1)
f_lin[1][1] = (x,p) -> basemodel(x,p)
for n = 2:npar+1
    aux = filter(x->sum(x) == n-1, func_map)
    f_lin[n] = Vector{Function}(undef, length(aux))
    k=1
    for a in aux 
        #vectorized function 
        f_lin[n][k] = (x,p) -> basemodel(x,p) .+ sum([p[i+4] for i=1:(n-1)] .* (fill(x,n-1) .|> func_list[a]))
        k = k+1
    end
end

f_non_lin = Vector{Vector{Function}}(undef, npar) #f[i][j]-> i: number of extra parameters, j: combinations
n_non_lin_param = Vector(5:npar+4) #param for each non-linear functions
for n = 1:npar
    aux = filter(x->sum(x) == n, func_map)
    f_non_lin[n] = Vector{Function}(undef, length(aux))
    k=1
    for a in aux 
        #vectorized function 
        f_non_lin[n][k] = (x,p) -> basemodel(x,p) .* (1 .+ sum([p[i+4] for i=1:(n)] .* (fill(x,n) .|> func_list[a])))
        k = k+1
    end
end



f_basemodel(x,p) = p[1] .+ p[2] .* x[:,2] .+ p[3] ./ sqrt.(x[:,3]) .+ p[4] .* x[:,1]
f_a2phi2_2(x) = x[:,1] .* x[:,2].^2              #a^2/8t0 
f_a2phi2(x) = x[:,1] .* x[:,2]      # phi2*a^2/8t0
f_a2phih2(x) = x[:,1] ./ x[:,3]  # 1/phih*a^2/8t0          
f_a3phih3(x) = x[:,1].^(4) ./ x[:,3].^2  #1/phih^3/2*(a^2/8t0)^3/2          
f_a3cutoff(x) = x[:,1].^(3/2)   #(a^2/8t0)^3/2

f_func_list = [f_a2phi2_2, f_a2phi2, f_a2phih2, f_a3phih3, f_a3cutoff]
f_npar = 5 # number of extra parameters 
f_f_lin = Vector{Vector{Function}}(undef, npar+1) #f[i][j]-> i: number of extra parameters, j: combinations
f_f_lin[1] = Vector{Function}(undef, 1)
f_f_lin[1][1] = (x,p) -> basemodel(x,p)
for n = 2:npar+1
    aux = filter(x->sum(x) == n-1, func_map)
    f_f_lin[n] = Vector{Function}(undef, length(aux))
    k=1
    for a in aux 
        #vectorized function 
        f_f_lin[n][k] = (x,p) -> f_basemodel(x,p) .+ sum([p[i+4] for i=1:(n-1)] .* (fill(x,n-1) .|> f_func_list[a]))
        k = k+1
    end
end

f_f_non_lin = Vector{Vector{Function}}(undef, npar) #f[i][j]-> i: number of extra parameters, j: combinations
n_non_lin_param = Vector(5:npar+4) #param for each non-linear functions
for n = 1:npar
    aux = filter(x->sum(x) == n, func_map)
    f_f_non_lin[n] = Vector{Function}(undef, length(aux))
    k=1
    for a in aux 
        #vectorized function 
        f_f_non_lin[n][k] = (x,p) -> f_basemodel(x,p) .* (1 .+ sum([p[i+4] for i=1:(n)] .* (fill(x,n) .|> f_func_list[a])))
        k = k+1
    end
end

n_param = n_lin_param
append!(n_param, n_non_lin_param)
f = f_lin
append!(f, f_non_lin)
f_dec = f_f_lin 
append!(f_dec, f_f_non_lin)